@@ -5,14 +5,14 @@
 // It is a module that executes the script written by C language. 
 // Please see http://csp.b-rabbit.com/ in detail. 
 //
-// author  : kumatiki
+// author  : fuji85
 //
 // Copyright (C) 2011 mod_csp project
 // License : Apache Licence 2.0
 //
-// version : 0.0.7
+// version : 0.1.0
 //
-// Last modification by kumatiki
+// Last modification by fuji85
 //
 ////////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -79,8 +79,6 @@ struct _param {
 //		include
 ////////////////////////////////////////////////////////////////////////////////////////////////
 
-# 1 "growmem.c" 1
-# 9 "growmem.c"
 typedef struct _growMem {
  apr_pool_t *p;
  char * mem;
@@ -135,7 +133,6 @@ static int gm_getsize(GROWMEM * gm){
  return gm->index;
 }
 # 8 "include.c" 2
-
 # 1 "file.c" 1
 
 
@@ -159,14 +156,35 @@ static char * fileRead(apr_pool_t * p,char * filename,int * s)
  * s = size;
  return buffer;
 }
-# 9 "include.c" 2
 
+
+static int fileWrite(apr_pool_t * p,char * buf,char * filename,int size)
+{
+ apr_file_t *fpw;
+
+ apr_status_t rv = apr_file_open(&fpw, filename, APR_WRITE|APR_CREATE|APR_TRUNCATE|APR_FOPEN_BINARY, APR_OS_DEFAULT , p);
+ if(rv!=APR_SUCCESS)return CSPNG;
+ apr_file_write(fpw,buf,&size);
+ apr_file_close(fpw);
+
+ return CSPOK;
+}
+
+int filecopy(apr_pool_t * p,char * srcfile,char * destfile){
+ int size;
+ char * tmp = fileRead(p,srcfile,&size);
+ return fileWrite(p,tmp,destfile,size);
+}
+# 9 "include.c" 2
 # 1 "param.c" 1
-# 10 "param.c"
+# 15 "param.c"
 struct csp_config {
  char * dir;
  char * iparam;
  char * lparam;
+
+
+ int htso_use;
  int timeout;
 };
 
@@ -180,6 +198,9 @@ static void * create_per_dir_config (apr_pool_t *pool, char *arg)
  cfg->lparam="";
  cfg->iparam="";
  cfg->timeout = 5;
+
+
+ cfg->htso_use = 0;
  return buf;
 }
 
@@ -201,12 +222,21 @@ static const char *cmd_csp_timeout (cmd_parms *parms, void *mconfig, char *arg){
  return 0;
 }
 
-
 static const char * cmd_csp_lparam (cmd_parms *parms, void *mconfig, char *arg){
  struct csp_config *cfg = (struct csp_config*)mconfig;
  cfg->lparam = apr_pstrdup(parms->pool,arg);
  return 0;
 }
+# 78 "param.c"
+static const char *cmd_htso_use (cmd_parms *parms, void *mconfig, char *arg){
+ struct csp_config *cfg = (struct csp_config*)mconfig;
+ int tmp = atoi(arg);
+ if(tmp < 0)tmp = 0;
+ if(tmp > 2)tmp = 0;
+ cfg->htso_use = tmp;
+ return 0;
+}
+
 
 static const char * cmd_csp_iparam (cmd_parms *parms, void *mconfig, char *arg){
  struct csp_config *cfg = (struct csp_config*)mconfig;
@@ -222,12 +252,16 @@ typedef const char *(*CMD_HAND_TYPE) (cmd_parms*,void*);
 static const command_rec csp_cmds[] = {
  AP_INIT_TAKE1("csp_dir", cmd_csp_dir, NULL, OR_ALL, "set csp module directory"),
  AP_INIT_TAKE1("csp_timeout", cmd_csp_timeout, NULL, OR_ALL, "set csp timeout"),
+
+
+
+ AP_INIT_TAKE1("csp_htso_use", cmd_htso_use, NULL, OR_ALL, "use Shared Object at csp root folder"),
+
  AP_INIT_RAW_ARGS("csp_lparam", cmd_csp_lparam, NULL, OR_ALL, "set csp default lparam"),
  AP_INIT_RAW_ARGS("csp_iparam", cmd_csp_iparam, NULL, OR_ALL, "set csp default iparam"),
  {0},
 };
 # 10 "include.c" 2
-
 # 1 "cookie.c" 1
 # 10 "cookie.c"
 static void setCookie(request_rec *r,char * args1,char * args2){
@@ -268,7 +302,6 @@ static apr_table_t * getCOOKIE(request_rec* r) {
     return cookei_params;
 }
 # 11 "include.c" 2
-
 # 1 "getpost.c" 1
 # 10 "getpost.c"
 static apr_table_t * getGET(request_rec* r) {
@@ -347,7 +380,6 @@ static apr_table_t * getPOST(request_rec* r) {
     return NULL;
 }
 # 12 "include.c" 2
-
 # 1 "trans.c" 1
 # 10 "trans.c"
 static int includetenkai(apr_pool_t * p,GROWMEM * buf,char * filename,char * dstdir,request_rec *r)
@@ -531,7 +563,6 @@ static int csptoc(apr_pool_t *p, GROWMEM * ten,char * dstcfile,char * dstdir,GRO
  apr_file_open(&fpw, dstcfile, APR_WRITE|APR_CREATE|APR_TRUNCATE|APR_FOPEN_BINARY, APR_OS_DEFAULT , p);
 
  apr_file_puts(
-
 # 1 "head.h" 1
 "////////////////////////////////////////////////////////////////////////////////////////////////\n"
 "//\n"
@@ -626,13 +657,13 @@ static int csptoc(apr_pool_t *p, GROWMEM * ten,char * dstcfile,char * dstdir,GRO
 "///////////////////////////////////////////////////////////////////////\n"
 "//\n"
 "///////////////////////////////////////////////////////////////////////\n"
-"static request_rec * __r__;\n"
-"static apr_pool_t * __p__;\n"
-"static apr_table_t * __get__;\n"
-"static apr_table_t * __post__;\n"
-"static apr_table_t * __cookie__;\n"
-"static 	int * __session_index__;\n"
-"static struct _sessionTable * __session__;\n"
+"request_rec * __r__;\n"
+"apr_pool_t * __p__;\n"
+"apr_table_t * __get__;\n"
+"apr_table_t * __post__;\n"
+"apr_table_t * __cookie__;\n"
+"int * __session_index__;\n"
+"struct _sessionTable * __session__;\n"
 "\n"
 "//setjump as core dump \n"
 "jmp_buf jbuf;\n"
@@ -963,7 +994,6 @@ static int csptoc(apr_pool_t *p, GROWMEM * ten,char * dstcfile,char * dstdir,GRO
  return 0;
 }
 # 13 "include.c" 2
-
 # 1 "session.c" 1
 # 10 "session.c"
 struct _sessionTable {
@@ -1084,6 +1114,25 @@ void errordispay(request_rec *r,char * title,char *contents){
 }
 
 
+
+
+
+apr_time_t getTimeStamp(char * filename,apr_pool_t *p){
+	apr_file_t * fpr = NULL;
+	apr_time_t  mtime = 0l;
+	apr_finfo_t finfo;
+	//I want to know the time stamp without opening 
+    apr_file_open(&fpr, filename, APR_READ, APR_OS_DEFAULT , p);
+	if(fpr!=NULL){
+	    apr_file_info_get(&finfo, APR_FINFO_CTIME, fpr);
+		mtime = finfo.ctime;
+		apr_file_close(fpr);
+	}
+	return mtime;
+}
+
+
+
 ////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //		handler
@@ -1106,16 +1155,10 @@ static int csp_handler(request_rec *r)
 	struct csp_config * cfg;
 	cfg = ap_get_module_config(r->per_dir_config, &csp_module);
 
-//	ap_rprintf(r,"csp_dir = %s<br>\n",cfg->dir);
-//	ap_rprintf(r,"csp_timeout = %d<br>\n",cfg->timeout);
-//	ap_rprintf(r,"csp_lparam = %s<br>\n",cfg->lparam);
-//	ap_rprintf(r,"csp_iparam = %s<br>\n",cfg->iparam);
-//	return OK;
-
-
 // 		Error buffer
 //    GROWMEM * errbuffer = getGrowMem(p,1024);
 
+//	filename = /var/www/html/XXXXXX.csp
 	// make folder name by accessed file 
 	char * folder = apr_pstrdup(p,(const char * )r->filename);
 	tmp = strrchr(folder,'/');
@@ -1127,6 +1170,12 @@ static int csp_handler(request_rec *r)
 	tmp = strrchr(tmp,'/');
 	if(tmp!=(char * )NULL) filename = tmp + 1;
 
+  	// sofile
+	char * sofile = NULL;
+	sofile = apr_pstrdup(p,(const char * )filename);
+	tmp = strrchr(sofile,'.');
+	if(tmp!=(char * )NULL) { * (tmp + 1) = 's';* (tmp + 2) = 'o';* (tmp + 3) = '\0';}
+	char * atsofile = cat(folder,"/.ht",sofile);
 
 	//get GETS
     apr_table_t * get = getGET(r);
@@ -1155,82 +1204,111 @@ static int csp_handler(request_rec *r)
 	char * dstsofile = cat(tmpdir,"/lib",folder,"/lib",filename,".so");
 
 	//get maked time
-    apr_time_t  mtime1 = 0l;     //
-    apr_time_t  mtime2 = 0l;     //csp file
-
+    apr_time_t  mtime1;     //
+    apr_time_t  mtime2;     //csp file
+    apr_time_t  mtime3;     //at so file
 	apr_file_t * fpr = NULL;
-	apr_finfo_t finfo1;
-	apr_finfo_t finfo2;
 
-	//I want to know the time stamp without opening 
-    apr_file_open(&fpr, dstsofile, APR_READ, APR_OS_DEFAULT , p);
-	if(fpr!=NULL){
-	    apr_file_info_get(&finfo1, APR_FINFO_CTIME, fpr);
-		mtime1 = finfo1.ctime;
-		apr_file_close(fpr);
-	}
-
-    apr_file_open(&fpr, r->filename, APR_READ, APR_OS_DEFAULT , p);
-	if(fpr==NULL){
+	//I want to know the time stamp without opening ; if mtime1==0 then not exist;
+	mtime1 = getTimeStamp(dstsofile,p);
+	mtime2 = getTimeStamp(r->filename,p);
+	//atsofile exist ?
+	mtime3 = getTimeStamp(atsofile,p);
+	if(mtime2==0l && mtime3==0l)
 		return   HTTP_NOT_FOUND;
+
+	char * fname;
+
+	if(cfg->htso_use==2){
+		//so_use == 1 and .htXXXXX.so file exist
+		chdir(folder);
+		fname = atsofile;
+	} else {
+		//It copies it when there is an update. 
+		if(mtime2>mtime1){
+			//make folder 
+			apr_status_t admrans = apr_dir_make_recursive(dstdir,APR_OS_DEFAULT,p);
+			if(admrans!=APR_SUCCESS){
+				ap_rprintf(r,"csp_dir error : couldn't write");
+				return   OK;
+			}
+
+		   //trans (include develops)
+		    GROWMEM * incl = getGrowMem(p,1024);
+			int ret = includetenkai(p,incl,filename,folder,r);
+			if(ret==CSPERROR){
+				ap_rprintf(r,"<h2>include error</h2>");
+				return OK;
+			}
+
+		   //trans
+		    GROWMEM * binclude = getGrowMem(p,1024);
+	    	GROWMEM * blib = getGrowMem(p,1024);
+			csptoc(p,incl,dstcfile,dstdir,binclude,blib,r);
+
+			//compiles
+			chdir(folder);
+			int ccc = system(cat("apxs -Wc,-std=c99 ","-I ",folder," ",gm_getmem(binclude)," ",cfg->iparam," -c ",dstcfile," 2>",dstfile,".log"));
+			if(ccc!=0){
+	  			char * errorlog = (char * )apr_pcalloc(p,1024*10);
+	  			int len = 1024*10;
+			    apr_file_open(&fpr, cat(dstfile,".log"), APR_READ, APR_OS_DEFAULT , p);
+				apr_file_read(fpr,errorlog,&len);
+				apr_file_close(fpr);
+				errordispay(r,"compile error : check iparam or csp_iparam",errorlog);
+				return   OK;
+			}
+			ccc = system(cat("gcc -shared -Wl,-soname,lib",filename,".so -o ",dstsofile," ",dstofile," ",gm_getmem(blib)," ",cfg->lparam," 2>",dstfile,"1.log"));
+			if(ccc!=0){
+	  			char * errorlog = (char * )apr_pcalloc(p,1024*10);
+	  			int len = 1024*10;
+			    apr_file_open(&fpr, cat(dstfile,"1.log"), APR_READ, APR_OS_DEFAULT , p);
+				apr_file_read(fpr,errorlog,&len);
+				apr_file_close(fpr);
+				errordispay(r,"link error",errorlog);
+				return   OK;
+			}
+
+			ccc = system(cat("strip ",dstsofile));
+			if(ccc!=0){
+				errordispay(r,"strip error","");
+				return   OK;
+			}
+
+			//if so_make==1 then make copy to atfile from sofile.
+			if(cfg->htso_use==1){
+				if(filecopy(p,dstsofile,atsofile)==CSPNG){
+					ap_rprintf(r,"<h2>.so FILE WRITE ERROR</h2>");
+					return OK;
+				}
+
+
+			}
+	
+		} else 
+			chdir(folder);
+
+		if(cfg->htso_use==0){
+			fname = dstsofile;
+		} else {
+			if(mtime3!=0l){
+				fname = atsofile;
+			} else {
+				// look at htsofile again
+				mtime3 = getTimeStamp(atsofile,p);
+				if(mtime3==0l){
+					fname = dstsofile;
+				} else {
+					fname = atsofile;
+				}
+			}
+		}
 	}
-    apr_file_info_get(&finfo2, APR_FINFO_CTIME, fpr);
-	mtime2 = finfo2.ctime;
-	apr_file_close(fpr);
-
-	//It copies it when there is an update. 
-	if(mtime2>mtime1){
-		//make folder 
-		apr_status_t admrans = apr_dir_make_recursive(dstdir,APR_OS_DEFAULT,p);
-		if(admrans!=APR_SUCCESS){
-			ap_rprintf(r,"csp_dir error : couldn't write");
-			return   OK;
-		}
-
-	   //trans (include develops)
-	    GROWMEM * incl = getGrowMem(p,1024);
-		int ret = includetenkai(p,incl,filename,folder,r);
-		if(ret==CSPERROR){
-			ap_rprintf(r,"<h2>include error</h2>");
-			return OK;
-		}
-
-
-	   //trans
-	    GROWMEM * binclude = getGrowMem(p,1024);
-    	GROWMEM * blib = getGrowMem(p,1024);
-		csptoc(p,incl,dstcfile,dstdir,binclude,blib,r);
-
-		//compiles
-		chdir(folder);
-		int ccc = system(cat("apxs -Wc,-std=c99 ","-I ",folder," ",gm_getmem(binclude)," ",cfg->iparam," -c ",dstcfile," 2>",dstfile,".log"));
-		if(ccc!=0){
-  			char * errorlog = (char * )apr_pcalloc(p,1024*10);
-  			int len = 1024*10;
-		    apr_file_open(&fpr, cat(dstfile,".log"), APR_READ, APR_OS_DEFAULT , p);
-			apr_file_read(fpr,errorlog,&len);
-			apr_file_close(fpr);
-			errordispay(r,"compile error : check iparam or csp_iparam",errorlog);
-			return   OK;
-		}
-		ccc = system(cat("gcc -shared -Wl,-soname,lib",filename,".so -o ",dstdir,"lib",filename,".so  ",dstofile," ",gm_getmem(blib)," ",cfg->lparam," 2>",dstfile,"1.log"));
-		if(ccc!=0){
-  			char * errorlog = (char * )apr_pcalloc(p,1024*10);
-  			int len = 1024*10;
-		    apr_file_open(&fpr, cat(dstfile,"1.log"), APR_READ, APR_OS_DEFAULT , p);
-			apr_file_read(fpr,errorlog,&len);
-			apr_file_close(fpr);
-			errordispay(r,"link error",errorlog);
-			return   OK;
-		}
-	} else 
-		chdir(folder);
 
 	//go go go
 	apr_dso_handle_t * dso_h = NULL;
 	apr_status_t rv;
 	int st = 0;
-	char * fname = cat(dstdir,"lib",filename,".so");
 
 	//set parameter 
 	struct _param param  = {
